@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2003 Anne-Marie Mahfouf <annma@kde.org>
+ * Copyright (C) 2001-2005 Anne-Marie Mahfouf <annma@kde.org>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of version 2 of the GNU General Public
@@ -15,8 +15,8 @@
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-#ifndef _KHANGMAN_H_
-#define _KHANGMAN_H_
+#ifndef KHANGMAN_H
+#define KHANGMAN_H
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -34,6 +34,7 @@ class KToggleToolBarAction;
 class QPopupMenu;
 class KToolBar;
 class KHNewStuff;
+class advanced;
 
 /**
  * This class serves as the main window for KHangMan.  It handles the
@@ -68,12 +69,10 @@ public:
     void setLanguages();
     ///Build the Level combobox menu dynamically depending of the data for each language
     void loadDataFiles();
-    ///Action for building the Languages menu
-    KSelectAction *langAct;
+    
+    advanced *mAdvanced;
     
     QStringList allData;
-    ///Build the Languages menu
-    void setupLangMenu();   
     /**
     Update the text in the Statusbar
     @param text the text that will be written in the statusbar
@@ -84,8 +83,6 @@ public:
 private:
     ///hold the current level
     QString levelString;
-    ///hold the current mode
-    QString modeString;
     ///the language used in KDE for the user
     QString userLanguage;
     ///the different data files in each language dir
@@ -126,14 +123,10 @@ private slots:
     @param newLanguage the index of the new selected language in the menu
     */
     void changeLanguage(int newLanguage);
-    ///Convenient slot to call changeLanguage slot
-    void slotLanguage();
     /**Set the current language in the view and update the statusbar
     @param lang the code of the currently selected language
     */
-    void setLanguage(QString lang);
-    ///When Transparent Pictures is checked/unchecked in Settings menu, go to this slot
-    void slotTransparent();
+    void setLanguage(const QString& lang);
     /**
     Slot to toggle the full screen mode	
     @param set the state of the Full Screen Mode ToggleMenu, true if toggled
@@ -162,6 +155,8 @@ private slots:
     ///Set a bool to true for languages that allow Typing Accented Letters
     void setAccentBool();
     
+    void setLevel();
+    
 private:
     ///Main view
     KHangManView *m_view;
@@ -182,4 +177,4 @@ private:
     
 };
 
-#endif // _KHANGMAN_H_
+#endif // KHANGMAN_H
