@@ -1,4 +1,3 @@
-// curve_data.cc
 // Copyright (C)  2002  Dominique Devriese <devriese@kde.org>
 
 // This program is free software; you can redistribute it and/or
@@ -17,6 +16,7 @@
 // 02111-1307, USA.
 
 #include "curve_imp.h"
+#include "../misc/coordinate.h"
 
 const ObjectImpType* CurveImp::stype()
 {
@@ -34,3 +34,8 @@ const ObjectImpType* CurveImp::stype()
     );
   return &t;
 }
+
+Coordinate CurveImp::attachPoint() const
+{
+  return Coordinate::invalidCoord();
+}
