@@ -1,4 +1,3 @@
-// transform_types.cc
 // Copyright (C)  2003  Dominique Devriese <devriese@kde.org>
 
 // This program is free software; you can redistribute it and/or
@@ -22,6 +21,7 @@
 #include "point_imp.h"
 #include "line_imp.h"
 #include "other_imp.h"
+#include "polygon_imp.h"
 #include "../misc/coordinate.h"
 #include "../misc/kigtransform.h"
 
@@ -206,6 +206,46 @@ ObjectImp* ScalingOverCenterType::calc( const Args& args, const KigDocument& ) c
   return args[0]->transform( Transformation::scalingOverPoint( ratio, center ) );
 }
 
+static const ArgsParser::spec argsspecScalingOverCenter2[] =
+{
+  { ObjectImp::stype(), I18N_NOOP( "Scale this object" ),
+    I18N_NOOP( "Select the object to scale..." ), false },
+  { PointImp::stype(), I18N_NOOP( "Scale with this center" ),
+    I18N_NOOP( "Select the center point of the scaling..." ), false },
+  { SegmentImp::stype(), I18N_NOOP( "Scale the length of this segment..." ),
+    I18N_NOOP( "Select the first of two segments whose ratio is the factor of the scaling..." ), false },
+  { SegmentImp::stype(), I18N_NOOP( "...to the length of this other segment" ),
+    I18N_NOOP( "Select the second of two segments whose ratio is the factor of the scaling..." ), false }
+};
+
+KIG_INSTANTIATE_OBJECT_TYPE_INSTANCE( ScalingOverCenter2Type )
+
+ScalingOverCenter2Type::ScalingOverCenter2Type()
+  : ArgsParserObjectType( "ScalingOverCenter2", argsspecScalingOverCenter2, 4 )
+{
+}
+
+ScalingOverCenter2Type::~ScalingOverCenter2Type()
+{
+}
+
+const ScalingOverCenter2Type* ScalingOverCenter2Type::instance()
+{
+  static const ScalingOverCenter2Type t;
+  return &t;
+}
+
+ObjectImp* ScalingOverCenter2Type::calc( const Args& args, const KigDocument& ) const
+{
+  if ( ! margsparser.checkArgs( args ) ) return new InvalidImp;
+
+  Coordinate center = static_cast<const PointImp*>( args[1] )->coordinate();
+  double ratio = static_cast<const SegmentImp*>( args[3] )->length()/
+                 static_cast<const SegmentImp*>( args[2] )->length();
+
+  return args[0]->transform( Transformation::scalingOverPoint( ratio, center ) );
+}
+
 static const ArgsParser::spec argsspecScalingOverLine[] =
 {
   { ObjectImp::stype(), I18N_NOOP( "Scale this object" ), I18N_NOOP( "Select the object to scale" ), false },
@@ -240,6 +280,42 @@ ObjectImp* ScalingOverLineType::calc( const Args& args, const KigDocument& ) con
   return args[0]->transform( Transformation::scalingOverLine( ratio, line ) );
 }
 
+static const ArgsParser::spec argsspecScalingOverLine2[] =
+{
+  { ObjectImp::stype(), I18N_NOOP( "Scale this object" ), I18N_NOOP( "Select the object to scale" ), false },
+  { AbstractLineImp::stype(), I18N_NOOP( "Scale over this line" ), I18N_NOOP( "Select the line to scale over" ), false },
+  { SegmentImp::stype(), I18N_NOOP( "Scale the length of this segment..." ), I18N_NOOP( "Select the first of two segments whose ratio is the factor for the scaling" ), false },
+  { SegmentImp::stype(), I18N_NOOP( "...to the length of this segment" ), I18N_NOOP( "Select the second of two segments whose ratio is the factor for the scaling" ), false }
+};
+
+KIG_INSTANTIATE_OBJECT_TYPE_INSTANCE( ScalingOverLine2Type )
+
+ScalingOverLine2Type::ScalingOverLine2Type()
+  : ArgsParserObjectType( "ScalingOverLine2", argsspecScalingOverLine2, 4 )
+{
+}
+
+ScalingOverLine2Type::~ScalingOverLine2Type()
+{
+}
+
+const ScalingOverLine2Type* ScalingOverLine2Type::instance()
+{
+  static const ScalingOverLine2Type t;
+  return &t;
+}
+
+ObjectImp* ScalingOverLine2Type::calc( const Args& args, const KigDocument& ) const
+{
+  if ( ! margsparser.checkArgs( args ) ) return new InvalidImp;
+
+  LineData line = static_cast<const AbstractLineImp*>( args[1] )->data();
+  double ratio = static_cast<const SegmentImp*>( args[3] )->length()/
+                 static_cast<const SegmentImp*>( args[2] )->length();
+
+  return args[0]->transform( Transformation::scalingOverLine( ratio, line ) );
+}
+
 static const ArgsParser::spec argsspecProjectiveRotation[] =
 {
   { ObjectImp::stype(), I18N_NOOP( "Projectively rotate this object" ), I18N_NOOP( "Select the object to rotate projectively" ), false },
@@ -314,6 +390,48 @@ ObjectImp* HarmonicHomologyType::calc( const Args& args, const KigDocument& ) co
     Transformation::harmonicHomology( center, axis ) );
 }
 
+static const ArgsParser::spec argsspecAffinityB2Tr[] =
+{
+  { ObjectImp::stype(), I18N_NOOP( "Generic affinity of this object" ),
+    I18N_NOOP( "Select the object to transform..." ), false },
+  { PolygonImp::stype3(), I18N_NOOP( "Map this triangle" ),
+    I18N_NOOP( "Select the triangle that has to be transformed onto a given triangle..." ), false },
+  { PolygonImp::stype3(), I18N_NOOP( "onto this other triangle" ),
+    I18N_NOOP( "Select the triangle that is the image by the affinity of the first triangle..." ), false }
+};
+
+KIG_INSTANTIATE_OBJECT_TYPE_INSTANCE( AffinityB2TrType )
+
+AffinityB2TrType::AffinityB2TrType()
+  : ArgsParserObjectType( "AffinityB2Tr", argsspecAffinityB2Tr, 3 )
+{
+}
+
+AffinityB2TrType::~AffinityB2TrType()
+{
+}
+
+const AffinityB2TrType* AffinityB2TrType::instance()
+{
+  static const AffinityB2TrType t;
+  return &t;
+}
+
+ObjectImp* AffinityB2TrType::calc( const Args& args, const KigDocument& ) const
+{
+  if ( ! margsparser.checkArgs( args ) ) return new InvalidImp;
+
+  std::vector<Coordinate> frompoints = static_cast<const PolygonImp*>( args[1] )->points();
+  std::vector<Coordinate> topoints = static_cast<const PolygonImp*>( args[2] )->points();
+
+  bool valid = true;
+  Transformation t = Transformation::affinityGI3P( frompoints, topoints,
+        valid );
+
+  if (valid == false) return new InvalidImp;
+  return args[0]->transform( t );
+}
+
 static const ArgsParser::spec argsspecAffinityGI3P[] =
 {
   { ObjectImp::stype(), I18N_NOOP( "Generic affinity of this object" ),
@@ -371,6 +489,48 @@ ObjectImp* AffinityGI3PType::calc( const Args& args, const KigDocument& ) const
   return args[0]->transform( t );
 }
 
+static const ArgsParser::spec argsspecProjectivityB2Qu[] =
+{
+  { ObjectImp::stype(), I18N_NOOP( "Generic projective transformation of this object" ),
+    I18N_NOOP( "Select the object to transform..." ), false },
+  { PolygonImp::stype4(), I18N_NOOP( "Map this quadrilateral" ),
+    I18N_NOOP( "Select the quadrilateral that has to be transformed onto a given quadrilateral..." ), false },
+  { PolygonImp::stype4(), I18N_NOOP( "onto this other quadrilateral" ),
+    I18N_NOOP( "Select the quadrilateral that is the image by the projective transformation of the first quadrilateral..." ), false }
+};
+
+KIG_INSTANTIATE_OBJECT_TYPE_INSTANCE( ProjectivityB2QuType )
+
+ProjectivityB2QuType::ProjectivityB2QuType()
+  : ArgsParserObjectType( "ProjectivityB2Qu", argsspecProjectivityB2Qu, 3 )
+{
+}
+
+ProjectivityB2QuType::~ProjectivityB2QuType()
+{
+}
+
+const ProjectivityB2QuType* ProjectivityB2QuType::instance()
+{
+  static const ProjectivityB2QuType t;
+  return &t;
+}
+
+ObjectImp* ProjectivityB2QuType::calc( const Args& args, const KigDocument& ) const
+{
+  if ( ! margsparser.checkArgs( args ) ) return new InvalidImp;
+
+  std::vector<Coordinate> frompoints = static_cast<const PolygonImp*>( args[1] )->points();
+  std::vector<Coordinate> topoints = static_cast<const PolygonImp*>( args[2] )->points();
+
+  bool valid = true;
+  Transformation t = Transformation::projectivityGI4P( frompoints, topoints,
+        valid );
+
+  if (valid == false) return new InvalidImp;
+  return args[0]->transform( t );
+}
+
 static const ArgsParser::spec argsspecProjectivityGI4P[] =
 {
   { ObjectImp::stype(), I18N_NOOP( "Generic projective transformation of this object" ),
@@ -495,11 +655,21 @@ const ObjectImpType* ScalingOverCenterType::resultId() const
   return ObjectImp::stype();
 }
 
+const ObjectImpType* ScalingOverCenter2Type::resultId() const
+{
+  return ObjectImp::stype();
+}
+
 const ObjectImpType* ScalingOverLineType::resultId() const
 {
   return ObjectImp::stype();
 }
 
+const ObjectImpType* ScalingOverLine2Type::resultId() const
+{
+  return ObjectImp::stype();
+}
+
 const ObjectImpType* ProjectiveRotationType::resultId() const
 {
   return ObjectImp::stype();
@@ -510,11 +680,21 @@ const ObjectImpType* HarmonicHomologyType::resultId() const
   return ObjectImp::stype();
 }
 
+const ObjectImpType* AffinityB2TrType::resultId() const
+{
+  return ObjectImp::stype();
+}
+
 const ObjectImpType* AffinityGI3PType::resultId() const
 {
   return ObjectImp::stype();
 }
 
+const ObjectImpType* ProjectivityB2QuType::resultId() const
+{
+  return ObjectImp::stype();
+}
+
 const ObjectImpType* ProjectivityGI4PType::resultId() const
 {
   return ObjectImp::stype();
@@ -550,11 +730,21 @@ bool ScalingOverCenterType::isTransform() const
   return true;
 }
 
+bool ScalingOverCenter2Type::isTransform() const
+{
+  return true;
+}
+
 bool ScalingOverLineType::isTransform() const
 {
   return true;
 }
 
+bool ScalingOverLine2Type::isTransform() const
+{
+  return true;
+}
+
 bool ProjectiveRotationType::isTransform() const
 {
   return true;
@@ -565,11 +755,21 @@ bool HarmonicHomologyType::isTransform() const
   return true;
 }
 
+bool AffinityB2TrType::isTransform() const
+{
+  return true;
+}
+
 bool AffinityGI3PType::isTransform() const
 {
   return true;
 }
 
+bool ProjectivityB2QuType::isTransform() const
+{
+  return true;
+}
+
 bool ProjectivityGI4PType::isTransform() const
 {
   return true;
