@@ -21,6 +21,8 @@
 #ifndef DEFINF_H
 #define DEFINF_H
 
+#include <qmap.h>
+#include <qstringlist.h>
 
 namespace Deinf
 {
@@ -32,12 +34,12 @@ struct Conjugation
 	unsigned int num;
 };
 
-class Index
+class KDE_EXPORT Index
 {
 	public:
 	Index();
 
-	QStringList deinflect(QString, QStringList &);
+	QStringList deinflect(const QString &, QStringList &);
 
 	private:
 	void load();
