@@ -35,19 +35,20 @@
 #include "finddialog.h"
 #include "ksutils.h"
 #include "infoboxes.h"
+#include "observinglist.h"
 #include "imagesequence.h"
 
 // to remove warnings
 #include "indimenu.h"
 #include "indidriver.h"
 
-KStars::KStars( bool doSplash ) :
+KStars::KStars( bool doSplash, bool clockrun, const QString &startdate ) :
 	DCOPObject("KStarsInterface"), KMainWindow(),
 	skymap(0), centralWidget(0), topLayout(0), viewToolBar(0), TimeStep(0),
 	actCoordSys(0), colorActionMenu(0), fovActionMenu(0),
 	AAVSODialog(0), findDialog(0), kns(0), 
 	indimenu(0), indidriver(0), indiseq(0),
-	DialogIsObsolete(false)
+	DialogIsObsolete(false), StartClockRunning( clockrun ), StartDateString( startdate )
 {
 	pd = new privatedata(this);
 
@@ -60,7 +61,7 @@ KStars::KStars( bool doSplash ) :
 				this, SLOT( datainitFinished(bool) ) );
 
 		pd->splash = new KStarsSplash(0, "Splash");
-		QObject::connect(pd->splash, SIGNAL( closeWindow() ), this, SLOT( closeWindow() ) );
+		QObject::connect(pd->splash, SIGNAL( closeWindow() ), kapp, SLOT( quit() ) );
 		QObject::connect(pd->kstarsData, SIGNAL( progressText(QString) ),
 				pd->splash, SLOT( setMessage(QString) ));
 		pd->splash->show();
@@ -71,6 +72,9 @@ KStars::KStars( bool doSplash ) :
 	//Set Geographic Location
 	pd->kstarsData->setLocationFromOptions();
 
+	//Pause the clock if the user gave the "--paused" arg
+	if ( ! StartClockRunning ) pd->kstarsData->clock()->stop();
+	
 	//set up Dark color scheme for application windows
 	DarkPalette = QPalette(QColor("red4"), QColor("DarkRed"));
 	DarkPalette.setColor( QPalette::Normal, QColorGroup::Base, QColor( "black" ) );
@@ -90,6 +94,8 @@ KStars::KStars( bool doSplash ) :
 	#else
 	kdDebug() << "Did not find glibc >= 2.1.  Will use ANSI-compliant sin()/cos() functions." << endl;
 	#endif
+
+	obsList = new ObservingList( this, this );
 }
 
 KStars::KStars( KStarsData* kd )
@@ -103,6 +109,8 @@ KStars::KStars( KStarsData* kd )
 
 	kd->clock()->start();
 
+    obsList = new ObservingList( this, this );
+
 	show();
 }
 
@@ -207,4 +215,7 @@ void KStars::mapGetsFocus() { map()->QWidget::setFocus(); }
 
 dms* KStars::LST() { return data()->LST; }
 
+ObservingList* KStars::observingList() { return obsList; }
+
 #include "kstars.moc"
+
