@@ -1,7 +1,5 @@
 /***************************************************************************
 
-    $Id: QueryManager.cpp,v 1.17 2004/02/23 15:02:55 annma Exp $
-
                              manage queries
 
     -----------------------------------------------------------------------
@@ -91,19 +89,20 @@ static t_type_rel InternalTypeRelations [] =
 };
 
 
-QString QueryManager::getSubType (QString type)
+QString QueryManager::getSubType (const QString & type)
 {
   int i;
-  if ((i = type.find(QM_TYPE_DIV)) >= 0) {
-    type.remove(0, i+1);
-    return type;
+  QString t = type;
+  if ((i = t.find(QM_TYPE_DIV)) >= 0) {
+    t.remove(0, i+1);
+    return t;
   }
   else
     return QString::null;
 }
 
 
-QString QueryManager::getMainType (QString type)
+QString QueryManager::getMainType (const QString & type)
 {
   int i;
   if ((i = type.find(QM_TYPE_DIV)) >= 0)
@@ -265,7 +264,7 @@ QuerySelection QueryManager::select(kvoctrainDoc *doc, int act_lesson,
      kvoctrainExpr *expr = doc->getEntry(i);
      unsigned int lessonno;
      if (altlearn)
-       lessonno = 1; //We only use a single array in Leitner style
+       lessonno = 0; //We only use a single array in Leitner style
      else
        lessonno = expr->getLesson();
 
@@ -607,7 +606,7 @@ bool QueryManager::compareDate (CompType type, time_t qd, time_t limit)
     case Within: erg = qd >= now-limit;           // newer date
     break;
 
-    case NotQueried: erg = qd == 0;    
+    case NotQueried: erg = qd == 0;
     break;
 
     default:
@@ -639,7 +638,7 @@ bool QueryManager::compareQuery (CompType type, int qgrade, int limit)
     case LessEqThan: erg = qgrade <= limit;   // sel has less count
     break;
 
-    case LessThan: erg = qgrade < limit;   
+    case LessThan: erg = qgrade < limit;
     break;
 
     default:
@@ -659,7 +658,7 @@ bool QueryManager::compareBad (CompType type, int bcount, int limit)
     case MoreThan: erg = bcount > limit;    // sel has higher bad count
     break;
 
-    case MoreEqThan: erg = bcount >= limit; 
+    case MoreEqThan: erg = bcount >= limit;
     break;
 
     case EqualTo: erg = bcount == limit;
@@ -671,7 +670,7 @@ bool QueryManager::compareBad (CompType type, int bcount, int limit)
     case LessEqThan: erg = bcount <= limit;   // sel has less count
     break;
 
-    case LessThan: erg = bcount < limit; 
+    case LessThan: erg = bcount < limit;
     break;
 
     default:
@@ -713,7 +712,7 @@ bool QueryManager::compareGrade (CompType type, grade_t qgrade, grade_t limit)
 }
 
 
-bool QueryManager::compareType (CompType type, QString exprtype, QString limit)
+bool QueryManager::compareType (CompType type, const QString & exprtype, const QString & limit)
 {
    bool erg = true;
    switch (type) {
@@ -780,17 +779,18 @@ void QueryManager::setTypeNames (vector<QString> names)
 }
 
 
-void QueryManager::setLessonItemStr (QString indices)
+void QueryManager::setLessonItemStr (const QString & indices)
 {
   int pos;
+  QString indices_copy = indices;
   lessonitems.clear();
-  while ((pos = indices.find(' ')) >= 0) {
-     QString s = indices.left(pos);
-     indices.remove(0, pos+1);
+  while ((pos = indices_copy.find(' ')) >= 0) {
+     QString s = indices_copy.left(pos);
+     indices_copy.remove(0, pos+1);
      lessonitems.push_back(s.toInt());
   }
-  if (indices.length() != 0) {
-    lessonitems.push_back(indices.toInt());
+  if (indices_copy.length() != 0) {
+    lessonitems.push_back(indices_copy.toInt());
   }
 }
 
